#include "GL_X3DShapeNode.h"
#include "GL_X3DGeometryNode.h"
#include "GL_X3DAppearanceNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace GL {

X3DShapeNode::X3DShapeNode()
: X3DChildNode(), _geometry(0), _appearance(0)
{
  defineTypeName("X3DShapeNode", "Shape");
}

X3DShapeNode::X3DShapeNode(const X3DShapeNode &N)
: X3DChildNode(N), _geometry(0), _appearance(0)
{
  setGeometry(static_cast<X3DGeometryNode *>(N._geometry));
  setAppearance(static_cast<X3DAppearanceNode *>(N._appearance));
}

X3DShapeNode::~X3DShapeNode()
{
  removeParentFromChild(this, _geometry);
  removeParentFromChild(this, _appearance);
}

void X3DShapeNode::setGeometry(X3DGeometryNode *geometry)
{
  removeParentFromChild(this, _geometry);
  _geometry = geometry;
  addParentToChild(this, _geometry);
}

void X3DShapeNode::setAppearance(X3DAppearanceNode *appearance)
{
  removeParentFromChild(this, _appearance);
  _appearance = appearance;
  addParentToChild(this, _appearance);
}

MFAbstractNode X3DShapeNode::getChildrenList() const
{
  MFAbstractNode childrenList;
  if (_geometry != 0)
    childrenList.push_back(_geometry);
  if (_appearance!= 0)    
    childrenList.push_back(_appearance);
    
  return childrenList; 
}

bool X3DShapeNode::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DGeometryNode *>(N) != 0)
  {
    if (_geometry != 0)
      return false;
      
    _geometry = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;
  }
  
  if (dynamic_cast<X3DAppearanceNode *>(N) != 0)
  {
    if (_appearance != 0)
      return false;
      
    _appearance = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;    
  }
  if (N != 0)
    cx3d << "GL::X3DShapeNode::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "GL::X3DShapeNode::addChild : cannot add a NULL node!" << endl;
    
  return false;
}

bool X3DShapeNode::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DGeometryNode *>(N) != 0)
  {
    removeParentFromChild(this, N);
    _geometry = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;
  }
  
  if (dynamic_cast<X3DAppearanceNode *>(N) != 0)
  {
    removeParentFromChild(this, N);
    _appearance = static_cast<SFNode>(N);
    addParentToChild(this, N);
    return true;    
  }
  if (N != 0)
    cx3d << "GL::X3DShapeNode::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "GL::X3DShapeNode::setChild : cannot add a NULL node!" << endl;
    
  return false;
}

bool X3DShapeNode::removeChild(const SFAbstractNode &N)
{
  if (_geometry == N)
  {
    removeParentFromChild(this, N);
    _geometry = 0;
    return true;
  } 
   
  if (_appearance == N)
  {
    removeParentFromChild(this, N);
    _appearance = 0;
    return true;
  } 
  
  return false;
}

}
}
