#include "X3D_CoordinateInterpolator2D.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

CoordinateInterpolator2D::CoordinateInterpolator2D()
  : X3DInterpolatorNode(),
    _keyValue(MFVec2f())
{
  defineTypeName("CoordinateInterpolator2D", "Interpolation");
}

CoordinateInterpolator2D::CoordinateInterpolator2D(const MFFloat &key,
                                                   const MFVec2f &keyValue)
  : X3DInterpolatorNode(key),
    _keyValue(keyValue)
{
  defineTypeName("CoordinateInterpolator2D", "Interpolation");
}

CoordinateInterpolator2D::CoordinateInterpolator2D(const CoordinateInterpolator2D &N)
  : X3DInterpolatorNode(N),
    _keyValue(N._keyValue)
{
}

SFAbstractNode CoordinateInterpolator2D::clone() const
{
  return new CoordinateInterpolator2D(*this);
}

CoordinateInterpolator2D::~CoordinateInterpolator2D()
{
}

void CoordinateInterpolator2D::setKeyValue(const MFVec2f &keyValue)
{
  _keyValue = keyValue;
}

void CoordinateInterpolator2D::load(const X3DFileElement *element)
{
  X3DInterpolatorNode::load(element);

  int index;
  index = element->getIndexAttribute("keyValue");
  if (index != -1)
    _keyValue = MFVec2f(element->getAttribute(index));
}

SFString &CoordinateInterpolator2D::write(SFString &output) const
{
  X3DInterpolatorNode::write(output);
  if (!_keyValue.empty())
    output += " keyValue=\"" + toSFString(_keyValue) + "\"";

  return output;  
}

}
}
