#include "X3D_NormalInterpolator.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

NormalInterpolator::NormalInterpolator()
  : X3DInterpolatorNode(),
    _keyValue(MFVec3f())
{
  defineTypeName("NormalInterpolator", "Interpolation");
}

NormalInterpolator::NormalInterpolator(const MFFloat &key,
                                           const MFVec3f &keyValue)
  : X3DInterpolatorNode(key),
    _keyValue(keyValue)
{
  defineTypeName("NormalInterpolator", "Interpolation");
}

NormalInterpolator::NormalInterpolator(const NormalInterpolator &N)
  : X3DInterpolatorNode(N),
    _keyValue(N._keyValue)
{
}

SFAbstractNode NormalInterpolator::clone() const
{
  return new NormalInterpolator(*this);
}

NormalInterpolator::~NormalInterpolator()
{
}

void NormalInterpolator::setKeyValue(const MFVec3f &keyValue)
{
  _keyValue = keyValue;
}

void NormalInterpolator::load(const X3DFileElement *element)
{
  X3DInterpolatorNode::load(element);

  int index;
  index = element->getIndexAttribute("keyValue");
  if (index != -1)
    _keyValue = MFVec3f(element->getAttribute(index));
}

SFString &NormalInterpolator::write(SFString &output) const
{
  X3DInterpolatorNode::write(output);
  
  if (!_keyValue.empty())
    output += " keyValue=\"" + toSFString(_keyValue) + "\"";

  return output;  
}

}
}
