#include "GL_Renderer.h"
#include "GL_DFSGraphTraversal.h"
#include "GL_RendererCoreVisitor.h"
#include "GL_RendererGroupingVisitor.h"
#include "GL_RendererRenderingVisitor.h"
#include "GL_RendererShapeVisitor.h"
#include "GL_RendererTexturingVisitor.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace GL {

Renderer::Renderer()
: X3DRenderer()
{
  stateVariables = GraphTraversal::getInstanceOf<RendererStateVariables>();

  graphTraversal = new GL::DFSGraphTraversal();
  graphTraversal->setComponentVisitor(new RendererCoreVisitor());
  graphTraversal->setComponentVisitor(new RendererGroupingVisitor());
  graphTraversal->setComponentVisitor(new RendererRenderingVisitor());
  graphTraversal->setComponentVisitor(new RendererShapeVisitor());
  graphTraversal->setComponentVisitor(new RendererTexturingVisitor());
}

Renderer::~Renderer()
{
  GraphTraversal::removeInstanceOf<RendererStateVariables>();

  delete graphTraversal;
}

void Renderer::render(X3DNode *N) const
{
  glDisable(GL_COLOR_MATERIAL);
  stateVariables->setSelectionMode(false);
  graphTraversal->traverse(N);
}

void Renderer::select(X3DNode *N, double x, double y) const
{
  stateVariables->setSelectionMode(true);
  stateVariables->initSelect(x, y);

  graphTraversal->traverse(N);

  stateVariables->finishSelect();
}


}
}
