#include "SFType.h"
#include "X3DActor.h"

#include <algorithm>
#include <iostream>

using namespace X3DTK;
using namespace std;

int SFType::count = 0;
MFType SFType::_typeList = MFType();  
list<X3DActor *> SFType::_actorList = list<X3DActor *>();
bool SFType::_functionDefinition = false;

void SFType::setParent(SFType *parent)
{
  _parent = parent;
}

void SFType::addOneReference()
{
  ++_refCount;
}

void SFType::addOneReferenceRecursive()
{
  ++_refCount;
  if (_parent != 0)
    _parent->addOneReferenceRecursive();
}

void SFType::removeOneReference(SFType *type)
{
  --type->_refCount;
  
  //removes the instance of the parents.
  if (type->_parent != 0)
  {
    if (type->_refCount == 0)  
      type->_parent->_childList.erase(type->getEncodedName());

    removeOneReference(type->_parent);
  }  
  
  if (type->_refCount == 0)  
  {
    //remove in the list.      
    _typeList.erase(type->getEncodedName());
    delete type;
  }
}

SFType *SFType::getTypeOfName(const SFString &sceneGraph, const SFString &component, const SFString &name)
{
  MFType::const_iterator it = _typeList.find(encodeKey(sceneGraph, component, name)); 
  if (it != _typeList.end())
    return (*it).second;
    
  //the SFType doesn't exist.
  return 0;  
}

SFType *SFType::getTypeOfId(int id)
{
  // iterate the SFTypes
  for (MFType::const_iterator it = _typeList.begin(); it != _typeList.end(); ++it)
    if ((*it).second->getId() == id)
      return (*it).second;
      
  return 0;      
}
void SFType::beginFunctionDefinition()
{
  _functionDefinition = true;
}

void SFType::endFunctionDefinition()
{
  _functionDefinition = false;
}

SFString SFType::encodeKey(const SFString &sceneGraph, const SFString &component, const SFString &SFType)
{
  return sceneGraph + "_" + component + "_" + SFType;
}

void SFType::recomputeIds()
{
}

void SFType::printDerivationTree()
{
  cout << "Derivation tree:" << endl;
  SFString tab("");
  //finding the nodes which have no parent.
  for (MFType::const_iterator it = _typeList.begin(); it != _typeList.end(); ++it)
  {
    if ((*it).second->_parent == 0)
    {
      (*it).second->printDerivationTree(tab);
    } 
  }    
  cout << "Derivation tree finished" << endl;
}

SFType::SFType(const SFString &name, const SFString &component, const SFString &sceneGraph)
: _name(name), _refCount(0)
{
  //cout << "build " << name <<  ": " << this << endl;

  //component update.
  _component = SFComponent::getComponent(sceneGraph, component);
  if (_component == 0)
    _component = new SFComponent(component, sceneGraph);

  //component referenced by a new SFType.
  _component->addOneReference();
}

SFType::~SFType()
{
  //cout << "destr SFType " << _name << endl;
  SFComponent::removeOneReference(_component);
  _childList.clear();
}

void SFType::defineTypeName(SFType **type, const SFString &name, const SFString &component, const SFString &sceneGraph)
{
  // Getting the name of the component.
  SFString tcomponent = component;
  if (tcomponent == "")
    tcomponent = (*type)->_component->getName();
  
  SFString tsceneGraph = sceneGraph;
  if ((tsceneGraph == "") && (*type != 0))
    tsceneGraph = (*type)->_component->_sceneGraph->getName();
     
  SFType *t = getTypeOfName(tsceneGraph, tcomponent, name);
  
  if (t == 0)
  {
    t = new SFType(name, tcomponent, tsceneGraph);
    t->_id = count;
    t->_parent = *type;  

    //adding new name.
    SFString nodeName = encodeKey(tsceneGraph, tcomponent, name);
       
    if (*type != 0)    
      (*type)->_childList[nodeName] = t;
      
      _typeList[nodeName] = t;

    if (!_functionDefinition)
    {
      ++count;
      //advertising actors
      for (list<X3DActor *>::iterator itActor = _actorList.begin(); itActor != _actorList.end(); ++itActor) 
        (*itActor)->addType(t);
    }
  }
 
  //the SFType t is referenced by a new Node.
  t->addOneReference();
  *type = t;
}

void SFType::addActor(X3DActor *actor)
{
  //updating proxy list
  _actorList.push_back(actor);
  
  //adding all SFTypes
  for (MFType::const_iterator it = _typeList.begin(); it != _typeList.end(); ++it)
    actor->addType((*it).second);
}

void SFType::removeActor(X3DActor *actor)
{
  _actorList.remove(actor);
}

void SFType::printDerivationTree(SFString tab) const
{
  cout << tab << _component->getSceneGraph()->getName() << "::" << _name << ": " << _refCount << ", " << _component->getName() << ": " << _component << ", "  << ": " << _component->_refCount << endl;
  tab += "  ";  

  for (MFType::const_iterator it = _childList.begin(); it != _childList.end(); ++it)
    (*it).second->printDerivationTree(tab);
}
