#include "X3DImageType.h"

#include <string.h>
#include <iostream>

using namespace std;

namespace X3DTK {

X3DImageType::X3DImageType()
: data(0), _pixelFormat(0), _width(0), _height(0)
{
}

X3DImageType::~X3DImageType()
{
  release();
}

void X3DImageType::resize(unsigned short width, unsigned short height)
{
  unsigned char *newData = new unsigned char[_pixelFormat*width*height];
  
  unsigned char *dataPtr = data;
  unsigned char *newDataPtr = newData;

  unsigned short rowStride = _pixelFormat*_width;
  unsigned short newRowStride = _pixelFormat*width;
  
  unsigned short copyRowStride = _pixelFormat*(width > _width ? _width : width);
  unsigned short copyHeight = (height > _height ? _height : height);
  
  for (unsigned short i = 0; i < copyHeight; ++i)
  {
    memcpy(newDataPtr, dataPtr, copyRowStride);
    dataPtr += rowStride;
    newDataPtr += newRowStride;
  }
  
  _width = width;
  _height = height;
  
  if (data != 0)
    delete [] data;

  data = newData;
}

void X3DImageType::resizeGL(float (& textureTransform)[16])
{
  // 1E-3 needed. Just try with width=128 and see !
  unsigned short newWidth  = 1<<(unsigned short)(1+log(_width -1+1E-3)/log(2.0));
  unsigned short newHeight = 1<<(unsigned short)(1+log(_height-1+1E-3)/log(2.0));

  //set the ratios for the next geometry node
  textureTransform[0] = (float)(_width)/(float)(newWidth);
  textureTransform[1] = 0.0f;
  textureTransform[2] = 0.0f;
  textureTransform[3] = 0.0f;
  
  textureTransform[4] = 0.0f;
  textureTransform[5] = (float)(_height)/(float)(newHeight);
  textureTransform[6] = 0.0f;
  textureTransform[7] = 0.0f;

  textureTransform[8] = 0.0f;
  textureTransform[9] = 0.0f;
  textureTransform[10] = 1.0f;
  textureTransform[11] = 0.0f;

  textureTransform[12] = 0.0f;
  textureTransform[13] = 0.0f;
  textureTransform[14] = 0.0f;
  textureTransform[15] = 1.0f;

  resize(newWidth, newHeight);
}

GLenum X3DImageType::getGLType() const
{
  switch (_pixelFormat)
  {
    case 0:
      return 0;
      break;
    case 1:
      return GL_LUMINANCE;
      break;  
    case 2:
      return GL_LUMINANCE_ALPHA;
      break;  
    case 3:
      return GL_RGB;
      break;  
    case 4:
      return GL_RGBA;
      break;  
  }
  return 0;
}

void X3DImageType::release()
{
  _pixelFormat = 0;
  _width = 0;
  _height = 0;
  
  if (data != 0)
  {
    delete [] data;
    data = 0;
  }  
}

bool X3DImageType::allocate(int pixelFormat, unsigned short width, unsigned short height)
{
  release();
  
  _pixelFormat = pixelFormat;
  _width = width;
  _height = height;
  
  data = new unsigned char[pixelFormat*width*height];
  
  return (data != 0);
}

}
